from .. helperFuncs import make_request as mr

class FreeToGame:
    """
    Class to interact with the Free To Game API.
    
    Attributes:
    - base_url: The base URL of the Free To Game API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://www.freetogame.com/api/"
        self.about = "The Free To Game API is a way to access programmatically the best free-to-play games and free MMO games."
        
    def get_docs_url(self):
        """
        Returns the URL for the Free To Game API documentation.
        
        Args:
        - None
        
        Returns:
        -string: The URL for the API docs.
        """
        return "https://www.freetogame.com/api-doc"
        
    def get_all_live_games(self, sort_by=None):
        """
        Returns the live games listed in the API.
        
        Args:
        - sort_by (str): Optional parameter by which to sort games if desired. Defaults to None, but supported values are release-date, popularity, alphabetical or relevance.
        
        Returns:
        - dict: A dictionary containing all the live games listed in the API.
        """
        if sort_by is not None:
            endpoint = f"games?sort-by={sort_by}"
        else:
            endpoint = "games"
        return mr.make_request(self.base_url+endpoint)
        
    def get_games_by_platform(self, platform, sort_by=None):
        """
        Returns a list of free games by platform.
        
        Args:
        - platform (str): The platform on which the game is played. Supported values are pc, browser, or all.
        - sort_by (str): Optional parameter by which to sort games if desired. Defaults to None, but supported values are release-date, popularity, alphabetical or relevance.
        
        Returns:
        - dict: A dictionary containing the free games played on the given platform.
        """
        if sort_by is not None:
            endpoint = f"games?platform={platform}&sort-by={sort_by}"
        else:
            endpoint = f"games?platform={platform}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_games_by_category(self, category, sort_by=None):
        """
        Returns a list of free games by category.
        
        Args:
        - category (str): The category of the game. Supported values are mmorpg, shooter, strategy, moba, racing, sports, social, sandbox, open-world, survival, pvp, pve, pixel, voxel, zombie, turn-based, first-person, third-Person, top-down, tank, space, sailing, side-scroller, superhero, permadeath, card, battle-royale, mmo, mmofps, mmotps, 3d, 2d, anime, fantasy, sci-fi, fighting, action-rpg, action, military, martial-arts, flight, low-spec, tower-defense, horror, mmorts.
        - sort_by (str): Optional parameter by which to sort games if desired. Defaults to None, but supported values are release-date, popularity, alphabetical or relevance.
        
        Returns:
        - dict: A dictionary containing the free games matching the given category.
        """
        if sort_by is not None:
            endpoint = f"games?category={category}&sort-by={sort_by}"
        else:
            endpoint = f"games?category={category}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_games_by_platform_and_category(self, platform, category, sort_by=None):
        """
        Returns a list of free games by category.
        
        Args:
        - platform (str): The platform on which the game is played. Supported values are pc, browser, or all.
        - category (str): The category of the game. Supported values are mmorpg, shooter, strategy, moba, racing, sports, social, sandbox, open-world, survival, pvp, pve, pixel, voxel, zombie, turn-based, first-person, third-Person, top-down, tank, space, sailing, side-scroller, superhero, permadeath, card, battle-royale, mmo, mmofps, mmotps, 3d, 2d, anime, fantasy, sci-fi, fighting, action-rpg, action, military, martial-arts, flight, low-spec, tower-defense, horror, mmorts.
        - sort_by (str): Optional parameter by which to sort games if desired. Defaults to None, but supported values are release-date, popularity, alphabetical or relevance.
        
        Returns:
        - dict: A dictionary containing the free games matching the given category.
        """
        if sort_by is not None:
            endpoint = f"games?platform={platform}&category={category}&sort-by={sort_by}"
        else:
            endpoint = f"games?platform={platform}&category={category}"
        return mr.make_request(self.base_url+endpoint)
        
    def filter_games(self, tag, platform="all", sort_by=None):
        """
        Returns a list of free games by category.
        
        Args:
        - tag (str): The filter tags, representing the categories of the filtered games, period delimited, no spaces. Supported values are mmorpg, shooter, strategy, moba, racing, sports, social, sandbox, open-world, survival, pvp, pve, pixel, voxel, zombie, turn-based, first-person, third-Person, top-down, tank, space, sailing, side-scroller, superhero, permadeath, card, battle-royale, mmo, mmofps, mmotps, 3d, 2d, anime, fantasy, sci-fi, fighting, action-rpg, action, military, martial-arts, flight, low-spec, tower-defense, horror, mmorts.
        - platform (str): Optional parameter representing the platform on which the games are played. Default is all, but other supported values are browser and pc.
        - sort_by (str): Optional parameter by which to sort games if desired. Defaults to None, but supported values are release-date, popularity, alphabetical or relevance.
        
        Returns:
        - dict: A dictionary containing the free games matching the given category.
        """
        if sort_by is not None:
            endpoint = f"filter?tag={tag}&platform={platform}&sort-by={sort_by}"
        else:
            endpoint = f"filter?tag={tag}&platform={platform}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_game_by_id(self, gameID):
        """
        Returns information for a single game matching the id.
        
        Args:
        - gameID (int): A number representing the unique id of the game as listed in the API.
        
        Returns:
        - dict: A dictionary containing information for the game matching the given id.
        """
        endpoint = f"game?id={gameID}"
        return mr.make_request(self.base_url+endpoint)
     
     
       
       
        
        


from .. helperFuncs import make_request as mr


class RestCountries:
    """
    A class to interact with the RestCountries API.
    
    Attributes:
    - base_url: The base URL of the RestCountries API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://restcountries.com/v3.1/"
        self.about = "REST Countries API is a simple REST API from RapidAPI that provides information about countries in the world In JSON format."
        
    def get_docs_url(self):
        """
        Returns url for API docs.
        
        Args:
        - None
        
        Returns:
        - string: Url for the API documentation.
        """
        return "https://restcountries.com/"
    
    def get_country_by_name(self, name):
        """
        Returns information about a country by its name.

        Args:
        - name (str): The name of the country.

        Returns:
        - dict: Information about the country, or an error if country not found.
        """
        endpoint = f"name/{name}"
        return mr.make_request(self.base_url+endpoint)
    
    def get_country_by_code(self, code):
        """
        Returns information about a country by its code (cca2, ccn3, cca3, or cioc).

        Args:
        - code (str): The country code.

        Returns:
        - dict: Information about the country, or an error if country not found.
        """
        endpoint = f"alpha/{code}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_country_by_currency(self, currency):
        """
        Returns information about countries that use a specific currency.

        Args:
        - currency (str): The currency code or name.

        Returns:
        - list: A list of dictionaries containing information about countries using the currency, or an error if no countries found.
        """
        endpoint = f"currency/{currency}"
        return mr.make_request(self.base_url+endpoint)

    def get_country_by_language(self, language):
        """
        Returns information about countries speaking a given language.

        Args:
        - language (str): The name of the spoken language of the country.

        Returns:
        - list: A list of dictionaries containing information about countries using the language, or an error if no countries found.
        """
        endpoint = f"lang/{language}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_country_by_capital(self, capital):
        """
        Returns information about a country with the given capital

        Args:
        - capital (str): The name of the capital city.

        Returns:
        - list: A list of dictionaries containing information about countries using the capital, or an error if no countries found.
        """
        endpoint = f"capital/{capital}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_country_by_region(self, region):
        """
        Returns information about countries based on a given region.

        Args:
        - region (str): The name of the region.

        Returns:
        - list: A list of dictionaries containing information about countries using the region, or an error if no country is found.
        """
        endpoint = f"region/{region}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_country_by_subregion(self, subregion):
        """
        Returns information about countries based on a given subregion.

        Args:
        - subregion (str): The name of the subregion.

        Returns:
        - list: A list of dictionaries containing information about countries using the region, or an error if no country is found.
        """
        endpoint = f"subregion/{subregion}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_country_by_translation(self, translation):
        """
        Returns information about countries based on a translation of the name.

        Args:
        - translation (str): The name in a given translation.

        Returns:
        - list: A list of dictionaries containing information about countries using the region, or an error if no country is found.
        """
        endpoint = f"translation/{translation}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_all_countries(self):
        """
        Returns information about all countries listed in the API.

        Args:
        - None

        Returns:
        - list: A list of dictionaries containing information about all the countries, or an error if no data is found.
        """
        endpoint = "all"
        return mr.make_request(self.base_url+endpoint)
    




from .. helperFuncs import make_request as mr

class BoredAPI:
    """
    A class to interact with Bored API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "http://www.boredapi.com/api/"
        self.about = "The Bored API helps you find things to do when you're bored. There are fields like the number of participants, activity type, and more that help you narrow down your results."
        
    def get_docs_url(self):
        """
        Returns the URL for the Bored API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://www.boredapi.com/documentation"
        
    def get_random_activity(self):
        """
        Returns a random activity with related information.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing an activity along with related information.
        """
        endpoint = "activity"
        return mr.make_request(self.base_url+endpoint)
        
    def get_activity_by_key(self, key):
        """
        Returns an activity by its key.
        
        Args:
        - key (int): A unique number identifying a specific activity listed in the API (from 1000001 to 9999998).
        
        Returns:
        - dict: A dictionary containing a specific activity along with related information.
        """
        endpoint = f"activity/?key={key}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_activity_by_type(self, activityType):
        """
        Returns a random activity by its type.
        
        Args:
        - activityType (str): The type of activity. Supported values are education, recreational, social, diy, charity, cooking, relaxation, music, busywork.
        
        Returns:
        - dict: A dictionary containing a random activity along with related information.
        """
        endpoint = f"activity/?type={activityType}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_activity_by_participants(self, numParticipants):
        """
        Returns a random activity by its number of participants.
        
        Args:
        - numParticipants (int): The number of participants required by the activity. Supported values are 1 to n.
        
        Returns:
        - dict: A dictionary containing a random activity along with related information.
        """
        endpoint = f"activity/?participants={numParticipants}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_activity_by_price(self, price):
        """
        Returns a random activity by a specified price.
        
        Args:
        - price (float): The price of the activity, as imagined within a range from 0.0 to 1.0.
        
        Returns:
        - dict: A dictionary containing a random activity along with related information.
        """
        endpoint = f"activity/?price={price}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_activity_within_price_range(self, minprice, maxprice):
        """
        Returns a random activity within a specified price range.
        
        Args:
        - minprice (float): The minimum price of the activity, starting from 0.0.
        - maxprice (float): The maximum price of the activity, no greater than 1.0.
        
        Returns:
        - dict: A dictionary containing a random activity along with related information.
        """
        endpoint = f"activity/?minprice={minprice}&maxprice={maxprice}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_activity_by_accessibility(self, accessibility):
        """
        Returns a random activity by a specified accessibility.
        
        Args:
        - accessibility (float): The accessibility of the activity, as imagined within a range from 0.0 to 1.0., with 0.0 being most accessible.
        
        Returns:
        - dict: A dictionary containing a random activity along with related information.
        """
        endpoint = f"activity/?accessibility={accessibility}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_activity_within_accessibility_range(self, minAccessibility, maxAccessibility):
        """
        Returns a random activity within a specified accessibility range.
        
        Args:
        - minAccessibility (float): The minimum accessibility of the activity, with 0.0 being the most accessible.
        - maxAccessibility (float): The maximum accessibility of the activity, no greater than 1.0.
        
        Returns:
        - dict: A dictionary containing a random activity along with related information.
        """
        endpoint = f"activity/?minaccessibility={minAccessibility}&maxaccessibility={maxAccessibility}"
        return mr.make_request(self.base_url+endpoint)
     
   
  
  
        
        
    
    
    
    
        


from .. helperFuncs import make_request as mr

class FilterLists:
    """
    A class for interacting with the FilterLists API.
    
    Attributes:
    - base_url: The base URL of the FilterLists API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "http://filterlists.com/api/directory/"
        self.about = "The FilterLists Directory API provides lists of filters used by AD blockers."
        
    def get_docs_url(self):
        """
        Returns the URL for the FilterLists API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://filterlists.com/api/#/"
        
    def get_languages(self):
        """
        Returns the languages targeted by the FilterLists.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing a list of languages targeted by the FilterLists.
        """
        endpoint = "languages"
        return mr.make_request(self.base_url+endpoint)
        
    def get_licenses(self):
        """
        Returns the licenses applied to the FilterLists.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing a list of licenses applied to the FilterLists.
        """
        endpoint = "licenses"
        return mr.make_request(self.base_url+endpoint)
        
    def get_lists(self):
        """
        Returns the FilterLists.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing the FilterLists.
        """
        endpoint = "lists"
        return mr.make_request(self.base_url+endpoint)
        
    def get_list_by_id(self, list_id):
        """
        Returns details of a specific FilterList matching the given id.
        
        Args:
        - list_id (int): A unique numerical identifier for a FilterList.
        
        Returns:
        - dict: A dictionary containing details of the FilterList.
        """
        endpoint = f"lists/{list_id}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_maintainers(self):
        """
        Returns the maintainers of the FilterLists.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing the maintainers of the FilterLists.
        """
        endpoint = "maintainers"
        return mr.make_request(self.base_url+endpoint)
        
    def get_software(self):
        """
        Returns the software that subscribes to the FilterLists.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing the software that subscribes to the FilterLists.
        """
        endpoint = "software"
        return mr.make_request(self.base_url+endpoint)
        
    def get_syntaxes(self):
        """
        Returns the syntaxes of the FilterLists.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing the syntaxes of the FilterLists.
        """
        endpoint = "syntaxes"
        return mr.make_request(self.base_url+endpoint)
        
    def get_tags(self):
        """
        Returns the tags of the FilterLists.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing the tags of the FilterLists.
        """
        endpoint = "tags"
        return mr.make_request(self.base_url+endpoint)
       
       
       
       
    
    
    
    


from .. helperFuncs import make_request as mr


class WallstreetBets:
    """
    A class to interact with the Wallstreet Bets API.
    
    Attributes:
    - base_url: The base URL of the Wallstreet Bets API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://tradestie.com/api/v1/apps/reddit"
        self.about = "This API gets the top 50 stocks discussed on the Reddit subreddit, Wallstreetbets"

    def get_docs_url(self):
        """
        Returns the URL for the Wallstreet Bets API documentation.
        
        Args:
        - None
        
        Returns:
        -string: The URL for the API docs.
        """
        return "https://tradestie.com/apps/reddit/api/"       
    
    def get_stock_sentiment_from_reddit(self):
        """
        Returns top 50 stocks discussed on Reddit subeddit - Wallstreetbets.

        Args:
        - None

        Returns:
        - list: List of dictionaries containing information about subreddit comments of top 50 stocks.
        """
        endpoint = None
        return mr.make_request(self.base_url)
    


from .. helperFuncs import make_request as mr

class OpenTriviaDB:
    """
    A class for interacting with the Open Trivia Database API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://opentdb.com/api.php?"
        self.about = "The Open Trivia Database provides a completely free JSON API to retrieve trivia questions for use in programming projects."
        
    def get_docs_url(self):
        """
        Returns the URL for the Open Trivia Database API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://opentdb.com/api_config.php"
        
    def get_random_trivia_questions(self, amount):
        """
        Returns the specified amount of random trivia questions of various difficulties and subjects, and the answers to those questions.
        
        Args:
        - amount (int): The number of trivia desired.
        
        Returns:
        - dict: A dictionary containing the specified amount of trivia questions along with relevant data and the answers.
        """
        endpoint = f"amount={amount}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_specified_trivia_questions(self, amount, category=None, difficulty=None, q_type=None):
        """
        Returns the specified amount of trivial questions matching the given parameters, along with relevant data and answers to the questions.
        
        Args:
        - amount (int): The number of trivia questions to return.
        - category (int): The category of trivia questions desired. Defaults to None, in which case any categories will be returned.
        - difficulty (str): The difficulty level of the trivia questions. Defaults to None, in which case each question can be any difficulty level. Supported values are easy, medium, and hard.
        - q_type (str): The type of questions. Defaults to None, in which case the types are random. Supported values are boolean (True or False) and multiple (Multiple Choice).
        """
        category_dict = {"general knowledge": 9, 
                         "books": 10,
                         "film": 11,
                         "music": 12,
                         "musicals and theatres": 13,
                         "television": 14,
                         "video games": 15,
                         "board games": 16,
                         "science and nature": 17,
                         "computers": 18,
                         "mathematics": 19,
                         "mythology": 20,
                         "sports": 21,
                         "geography": 22,
                         "history": 23,
                         "politics": 24,
                         "art": 25,
                         "celebrities": 26,
                         "animals": 27,
                         "vehicles": 28,
                         "comics": 29,
                         "gadgets": 30,
                         "japanese anime and manga": 31,
                         "cartoon and animations": 32
        }
        # returns any difficulty of any type of a specific amount and category
        if category is not None and difficulty is None and q_type is None:
            endpoint = f"amount={amount}&category={category_dict[category.lower()]}"
            return mr.make_request(self.base_url+endpoint)
            
        # returns any category and any type of a specific amount and difficulty 
        elif difficulty is not None and category is None and q_type is None:
            endpoint = f"amount={amount}&difficulty={difficulty.lower()}"
            return mr.make_request(self.base_url+endpoint)
            
        # returns any category and any difficulty of a specific amount and question type
        elif q_type is not None and category is None and difficulty is None:
            endpoint = f"amount={amount}&type={q_type.lower()}"
            return mr.make_request(self.base_url+endpoint)
            
        # returns any question type of a specific amount, category, and difficulty.
        elif category is not None and difficulty is not None and q_type is None:
            endpoint = f"amount={amount}&category={category_dict[category.lower()]}&difficulty={difficulty.lower()}"
            return mr.make_request(self.base_url+endpoint)
            
        # returns any category of a specific amount, difficulty, and question type.
        elif difficulty is not None and q_type is not None and category is None:
            endpoint = f"amount={amount}&difficulty={difficulty.lower()}&type={q_type.lower()}"
            return mr.make_request(self.base_url+endpoint)
            
        # returns any difficulty of a specific amount, category, and question type
        elif category is not None and q_type is not None and difficulty is None:
            endpoint = f"amount={amount}&category={category_dict[category.lower()]}&type={q_type.lower()}"
            return mr.make_request(self.base_url+endpoint)
            
        # returns random questions of a given amount (same as get_random_trivia_questions).
        else:
            return self.get_random_trivia_questions(amount=amount)       
                       


from .. helperFuncs import make_request as mr

class APIsGuru:
    """
    A class to interact with the APIs.guru API.
    
    Attributes:
    - base_url: The base url for the APIs.guru API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://api.apis.guru/v2/"
        self.about = "The APIs.guru API is a self-proclaimed Wikipedia for APIs, maintaining an Open API directory."
        
    def get_docs_url(self):
        """
        Returns the URL for the APIs.guru API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://apis.guru/api-doc/"
        
    def get_all_providers(self):
        """
        Returns a list of all the providers in the API directory.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing a list of all the providers in the directory.
        """
        endpoint = "providers.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_apis_by_provider(self, provider):
        """
        Returns a list of all the APIs for a particular provider.
        
        Args:
        - provider (str): The name of the api provider.
        
        Returns:
        - dict: A dictionary containing a list of all the APIs for a particular provider.
        """
        endpoint = f"{provider}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_service_names_by_provider(self, provider):
        """
        Returns a list of all the services for a particular provider.
        
        Args:
        - provider (str): The name of the provider.
        
        Returns:
        - dict: A dictionary containing a list of all the services for a particular provider.
        """
        endpoint = f"{provider}/services.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_particular_api_version_no_service_name(self, provider, api):
        """
        Returns the API entry for one specific version of an API where there is no service name.
        
        Args:
        - provider (str): The name of the provider.
        - api (str): The api.
        
        Returns:
        - dict: A dictionary containing the API entry for one specific version of an API where there is no service name.
        """
        endpoint = f"specs/{provider}/{api}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_particular_api_version_with_service_name(self, provider, service, api):
        """
        Returns the API entry for one specific version of an API where there is a s ervice name.
        
        Args:
        - provider (str): The name of the provider.
        - service (str): The name of the service.
        - api (str): The name of the API.
        
        Returns:
        - dict: A dictionary containing the API entry for one specific version of an API where there is a service name.
        """
        endpoint = f"specs/{provider}/{service}/{api}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_all_apis(self):
        """
        Returns a list of all the APIs in the API directory.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing a list of all the APIs in the directory.
        """
        endpoint = "list.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_basic_metrics(self):
        """
        Returns basic metrics for the entire API directory.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing basic metrics for the entire directory.
        """
        endpoint = "metrics.json"
        return mr.make_request(self.base_url+endpoint)
  
  
        
        
    
      
      
      
        


from .. helperFuncs import make_request as mr

class Microlink:
    """
    A class for interacting with the Microlink API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    
    def __init__(self):
        self.base_url = "https://api.microlink.io"
        self.about = "Microlink API provides a powerful API for automating any browser action."
        
    def get_docs_url(self):
        """
        Returns the URL for the Microlink API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://microlink.io/docs/api/getting-started/overview"
        
    def get_metadata_for_target_url(self, url):
        """
        Returns metadata for the target URL.
        
        Args:
        - url (str): The target URL.
        
        Returns:
        - dict: A dictionary containing metadata for the target URL.
        """
        params = {'url': url}
        return mr.make_request_with_params(self.base_url, params)
        
    def get_screenshot_of_target_url(self, url):
        """
        Returns the URL for a screenshot of the target URL.
        
        Args:
        - url (str): The target URL.
        
        Returns:
        - string: The URL for the screenshot of the target URL.
        """
        params = {'url': url, "screenshot": True}
        metadata = mr.make_request_with_params(self.base_url, params)
        return metadata["data"]["screenshot"]["url"]
        
    def get_pdf_of_target_url(self, url):
        """
        Returns the URL for a pdf of the target URL.
        
        Args:
        - url (str): The target URL.
        
        Returns:
        - string: The URL for the pdf of the target URL.
        """
        params = {'url': url, "pdf": True}
        metadata = mr.make_request_with_params(self.base_url, params)
        return metadata["data"]["pdf"]["url"]
        
    def get_color_palette_for_target_url(self, url):
        """
        Returns metadata containing the predominant color pallette for each image detected over the target URL.
        
        Args:
        - url (str): The target URL.
        
        Returns:
        - dict: A dictionary of metadata containing the predominant color pallette for each image detected over the target URL.
        """
        params = {'url': url, "palette": True}
        metadata = mr.make_request_with_params(self.base_url, params)
        return metadata
        
        

        
        


from .. helperFuncs import make_request as mr

class RandomUserGenerator:
    """
    A class to interact with the Random User Generator API.
    
    Attributes:
    - base_url: The base url of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://randomuser.me/api/"
        self.about = "The Random User Generator API is a free, open-source API for generating random user data, like Lorem Ipsum for people."
        
    def get_docs_url(self):
        """
        Returns the url for the Random User Generator API documentation.
        
        Args:
        - None
        
        Returns:
        - string: URL for API documentation.
        """
        return "https://randomuser.me/"
        
    def generate_random_user(self):
        """
        Returns randomly generated data for a random user.
        
        Args:
        - None
        
        Returns:
        -dict: Dictionary containing random information (such as name, DOB, SSN, address, etc) for a random user.
        """
        return mr.make_request(self.base_url)
        


from .. helperFuncs import make_request as mr

class Gutendex:
    """
    A class for interacting with the Gutendex API.
    
    Attributes:
    base_url: The base URL for the API.
    about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://gutendex.com/books/"
        self.about = "Gutendex is a JSON web API for Project Gutenberg ebook metadata."
        
    def get_docs_url(self):
        """
        Returns the URL for the Gutendex API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://gutendex.com/"
        
    def get_books(self, page=1, sort="popular", language="en"):
        """
        Returns a list of book metadata contained in the API.
        
        Args:
        - page (int): The page of the book list. Defaults to the first page.
        - sort (str): Sort option for the book metadata. The default is popular, based on number of downloads. Other supported values are ascending and descending.
        - language (str): The language in which the books are written. Defaults to en (English). Can take multiple comma separated (non-spaced) values.
       
        Returns:
        - dict: A dictionary containing the book metadata for the given page.
        """
        endpoint = f"?page={page}&sort={sort}&languages={language}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_book_by_id(self, book_id):
        """
        Returns metadata for a single book matching the given id.
        
        Args:
        - book_id (int): A unique integer identifying the book in the API database.
        
        Returns:
        - dict: A dictionary containing metadata for a single book.
        """
        endpoint = f"{book_id}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_books_by_ids(self, book_ids):
        """
        Returns metadata for a single book matching the given id.
        
        Args:
        - book_ids (str): Comma separated (no spaces) integers, written as a string, identifying the book in the API database.
        
        Returns:
        - dict: A dictionary containing metadata for a single book.
        """
        endpoint = f"?ids={book_ids}"
        return mr.make_request(self.base_url+endpoint)
        
    def search_books_by_author_and_title(self, author, title, sort="popular", language="en"):
        """
        Returns metadata on books from the given author and/or a give title.
        
        Args:
        - author (str): Any part of the author's name.
        - title (str): Any part of the book's title.
        - sort (str): Parameter by which to sort the data. Default is popular, and other supported values are ascending and descending.
        - language (str): The language in which the books are written. Defaults to en (English). Can take multiple comma separated (non-spaced) values.
        
        Returns:
        - dict: A dictionary containing metadata on books from the given author and/or given title. 
        """
        endpoint = f"?search={author}%20{title}&sort={sort}&languages={language}"
        print(self.base_url+endpoint)
        return mr.make_request(self.base_url+endpoint)
        
    def search_books_by_topic(self, topic, sort="popular", language="en"):
        """
        Returns metadata on books matching the given topic.
        
        Args:
        - topic (str): The topic to search.
        - sort (str): Parameter by which to sort the data. Default is popular, and other supported values are ascending and descending.
        - language (str): The language in which the books are written. Defaults to en (English). Can take multiple comma separated (non-spaced) values.
        
        Returns:
        - dict: A dictionary containing metadata for books matching the given topic.
        """
        endpoint = f"?topic={topic}&sort={sort}&languages={language}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_authors_within_date_range(self, author_year_start, author_year_end, sort="popular", language="en"):
        """
        Returns authors and metadata for their books within the specified year range.
        
        Args:
        - author_year_start (int): The year marking the beginning of range. Can be negative (for BC era) or positive.
        - author_year_end (int): The year marking the end of range. Can be negative (for BC era) or positive.
        - sort (str): Parameter by which to sort the data. Default is popular, and other supported values are ascending and descending.
        - language (str): The language in which the books are written. Defaults to en (English). Can take multiple comma separated (non-spaced) values.
        
        Returns:
        - dict: A dictionary containing a list of books whose authors lived within the specified range.
        """
        endpoint = f"?author_year_start={author_year_start}&author_year_end={author_year_end}&sort={sort}&languages={language}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_authors_before_specified_year(self, author_year_end, sort="popular", language="en"):
        """
        Returns authors and metadata for their books existing before the specified year.
        
        Args:
        - author_year_end (int): The year before which the author was alive. Can be negative (for BC era) or positive.
        - sort (str): Parameter by which to sort the data. Default is popular, and other supported values are ascending and descending.
        - language (str): The language in which the books are written. Defaults to en (English). Can take multiple comma separated (non-spaced) values.
        
        Returns:
        - dict: Returns a dictionary containing metadata for books by authors who were alive before the specified year.
        """
        endpoint = f"?author_year_end={author_year_end}&sort={sort}&languages={language}"
        return mr.make_request(self.base_url+endpoint)


from .. helperFuncs import make_request as mr

class UniversityDomainsAndNames:
    """
    A class to interact with the University Domains and Names API.
    
    Attributes:
    - base_url: The base url for the University Domains and Names API.
    - about: A short description of the API.
    """
    
    def __init__(self):
        self.base_url = "http://universities.hipolabs.com/search?"
        self.about = "This API accesses a list of universities and their domain names."
        
    def get_docs_url(self):
        """
        Returns the url for the University Domains and Names API documentation.
        
        Args:
        - None
        
        Returns:
        - string: URL for API documentation.
        """
        return "https://github.com/Hipo/university-domains-list"
        
    def get_university_name_and_domain(self, name, country, offset=None, limit=None):
        """
        Returns university names and domains for the given parameters.
        
        Args:
        - name (str): Whole or partial name of university in question.
        - country (str): Country of university in question.
        - offset (int): Optional pagination parameter to limit search. Defaults to None.
        - limit (int): Optional pagination parameter to limit number of results. Defaults to None.
        
        Returns:
        - dict: List containing universities matching parameters with their respective domains.
        """
        if offset is not None and limit is not None:
            endpoint = f"name={name}&country={country}&offset={offset}&limit={limit}"
        elif offset is not None:
            endpoint = f"name={name}&country={country}&offset={offset}"
        elif limit is not None:
            endpoint = f"name={name}&country={country}&limit={limit}"
        else:
            endpoint = f"name={name}&country={country}"
        return mr.make_request(self.base_url+endpoint)
        
    def update_list(self):
        """
        Forces a refresh of the API, since the API won't automatically update the dataset if it changes.
        
        Args:
        - None
        
        Returns:
        - dict: An updated, refreshed list of the university dataset.
        """
        endpoint = "update"
        return mr.make_request(self.base_url+endpoint)
        
        
        
      
        


import requests

from .. helperFuncs import make_request as mr

class LoremPicsum:
    """
    A class for interacting with the Lorem Picsum API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://picsum.photos/"
        self.about = "This is an API for getting placeholder images, a Lorem Ipsum for images."
        
    def get_docs_url(self):
        """
        Returns the URL for the Lorem Picsum API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://picsum.photos/"
        
    def get_random_image(self, width, height=None, grayscale=False, blur=None):
        """
        Returns the URL for to a random placeholder image with the given dimensions.
        
        Args:
        - width (int): The width of the image.
        - height (int): The height of the image. Defaults to None in which case the image is square.
        - grayscale (boolean): Optional argument for returning a grayscale picture. The default value is False.
        - blur (int): An option to return a blurred photo. Values range from 1 to 10.
        
        Returns:
        string: The URL of the placeholder image.
        """
        if height is not None:
            if grayscale:
                if blur is not None:
                    endpoint = f"{width}/{height}?grayscale&blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"{width}/{height}?grayscale"
                    return self.base_url+endpoint
            else:
                if blur is not None:
                    endpoint = f"{width}/{height}?blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"{width}/{height}"
                    return self.base_url+endpoint
        else:
            if grayscale:
                if blur is not None:
                    endpoint = f"{width}?grayscale&blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"{width}?grayscale"
                    return self.base_url+endpoint
            else:
                if blur is not None:
                    endpoint = f"{width}?blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"{width}"
                    return self.base_url+endpoint
            
    def get_image_by_id(self, image_id, width, height=None, grayscale=False, blur=None):
        """
        Returns the URL for a specific image matching the given id number.
        
        Args:
        - image_id (int): A unique numerical id matching a specific image.
        - width (int): The width of the image.
        - height (int): The height of the image. Defaults to None in which case the image is square.
        
        Returns:
        - string: The URL to the image matching the given parameters.
        """
        if height is not None:
            if grayscale:
                if blur is not None:
                    endpoint = f"id/{image_id}/{width}/{height}?grayscale&blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"id/{image_id}/{width}/{height}?grayscale"
                    return self.base_url+endpoint
            else:
                if blur is not None:
                    endpoint = f"id/{image_id}/{width}/{height}?blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"id/{image_id}/{width}/{height}"
                    return self.base_url+endpoint
        else:
            if grayscale:
                if blur is not None:
                    endpoint = f"id/{image_id}/{width}?grayscale&blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"id/{image_id}/{width}?grayscale"
                    return self.base_url+endpoint
            else:
                if blur is not None:
                    endpoint = f"id/{image_id}/{width}?blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"id/{image_id}/{width}"
                    return self.base_url+endpoint
            
            
    def get_static_random_image(self, seed, width, height=None, grayscale=False, blur=None):
        """
        Returns the URL for to a random placeholder image with the given dimensions.
        
        Args:
        - seed (str): A seed term to ensure the same random image is returned each time.
        - width (int): The width of the image.
        - height (int): The height of the image. Defaults to None in which case the image is square.
        
        Returns:
        string: The URL of the placeholder image.
        """
        if height is not None:
            if grayscale:
                if blur is not None:
                    endpoint = f"seed/{seed}/{width}/{height}?grayscale&blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"seed/{seed}/{width}/{height}?grayscale"
                    return self.base_url+endpoint
            else:
                if blur is not None:
                    endpoint = f"seed/{seed}/{width}/{height}?blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"seed/{seed}/{width}/{height}"
                    return self.base_url+endpoint
        else:
            if grayscale:
                if blur is not None:
                    endpoint = f"seed/{seed}/{width}?grayscale&blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"seed/{seed}/{width}?grayscale"
                    return self.base_url+endpoint
            else:
                if blur is not None:
                    endpoint = f"seed/{seed}/{width}?blur={blur}"
                    return self.base_url+endpoint
                else:
                    endpoint = f"seed/{seed}/{width}"
                    return self.base_url+endpoint
                    
    def list_images(self, page=1, limit=30):
        """
        Returns a list of images in the Lorem Picsum API.
        
        Args:
        - page (int): The page of images to return. Default is the first page.
        - limit (int): The number of items per page. Default is 30.
        
        Returns:
        - list: A list of images in the Lorem Picsum API.
        """
        endpoint = f"v2/list?page={page}&limit={limit}"
        content = requests.get(self.base_url+endpoint)
        return content.text
        
    def get_image_info_by_id(self, image_id):
        """
        Returns info about a specific image matching the given id.
        
        Args:
        - image_id (int): A number matching a specific image.
        
        Returns:
        - dict: A dictionary containing info about a specific image.
        """
        endpoint = f"id/{image_id}/info"
        return mr.make_request(self.base_url+endpoint)
        
    def get_image_info_by_seed(self, seed):
        """
        Returns info about a specific image matching the given seed.
        
        Args:
        - seed (str): The seed matching a specific image.
        
        Returns:
        - dict: A dictionary containing info about a specific image.
        """
        endpoint = f"seed/{seed}/info"
        return mr.make_request(self.base_url+endpoint)
   
            
        
        
        


from .. helperFuncs import make_request as mr

class NobelPrizeAPI:
    """
    A class to interact with the Nobel Prize API.
    
    Attributes:
    - base_url: The base url for the Nobel Prize API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://api.nobelprize.org/2.1/"
        self.about = "The Nobel Prize API returns all information about Laureates and Nobel Prizes."
        
    def get_docs_url(self):
        """
        Returns the URL for the Nobel Prize API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the Nobel Prize API docs.
        """
        return "https://www.nobelprize.org/about/developer-zone-2/"
        
    def get_nobel_prize_laureates(self):
        """
        Returns a list of Nobel Prize laureates with information about the winners.
        
        Args:
        - None
        
        Returns
        - dict: A dictionary containing a list of Nobel Prize laureates and relevant information about the winners.
        """
        endpoint = "laureates"
        return mr.make_request(self.base_url+endpoint)
        
    def get_all_nobel_prizes(self):
        """
        Returns a list of all the Nobel Prizes award along with information about the specific prize and winners.
        
        Args:
        - None
        
        Returns
        - dict: A dictionary containing a list of Nobel Prizes awarded and relevant information about the prize.
        """
        endpoint = "nobelPrizes"
        return mr.make_request(self.base_url+endpoint)
        
    def get_one_nobel_prize(self, category, year):
        """
        Returns a specific Nobel Prize and information about the specific prize and winners.
        
        Args:
        - category (str): The category of the prize awarded. Possible values are che (chemistry), eco (economic sciences), lit (literature), pea (peace), phy (physics), and med (medicine). Any other strings passed for category will return all the pieces for specified year.
        - year (int): The year the prize was awarded.
        
        Returns
        - dict: A dictionary containing a specific Nobel Prize awarded and relevant information about the prize.
        """
        endpoint = f"nobelPrize/{category.lower()}/{year}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_nobel_laureate_by_id(self, laureateID):
        """
        Returns a specific Nobel Prize and information about the specific prize and winners.
        
        Args:
        - laureateID (int): A unique identifier for a specific Nobel laureate.
        
        Returns
        - dict: A dictionary containing a specific Nobel Prize awarded and relevant information about the prize.
        """
        endpoint = f"laureate/{laureateID}"
        return mr.make_request(self.base_url+endpoint)
    
     
          
        
 


import requests
from requests.exceptions import HTTPError, Timeout, RequestException

def make_request(url, headers=None, payload=None):
    """
    Make an HTTP request to the specified URL with optional headers and payload.
    
    Args:
        url (str): The URL of the API endpoint.
        headers (dict, optional): Headers to be included in the request.
        payload (dict, optional): The payload to be sent in the request body.
        
    Returns:
        dict: A dictionary containing the response data or error information.
    """
    try:
        if payload is not None:
            response = requests.post(url, headers=headers, json=payload)
        else:
            response = requests.get(url, headers=headers)
        
        response.raise_for_status()  # Raise an exception for 4xx or 5xx status codes
        return response.json()  # Return JSON response
    except HTTPError as http_err:
        # Handle HTTP error
        return {"error": f"HTTP error occurred: {http_err}"}
    except Timeout:
        # Handle timeout error
        return {"error": "Request timed out."}
    except RequestException as req_err:
        # Handle other request exceptions
        return {"error": f"Request exception occurred: {req_err}"}
    except Exception as err:
        # Handle any other unexpected errors
        return {"error": f"An unexpected error occurred: {err}"}

def make_request_with_params(url, params):
    """
    Make a request to an API if the API call requires params.
    
    Args:
    - url (str): The url of the API.
    - params (dict): The params to be included in the request.
    
    Returns:
    - dict: A dictionary containing either the response data or an error message.
    """
    try:
        response = requests.get(url, params)
        return response.json()
    except HTTPError as http_err:
        # Handle HTTP error
        return {"error": f"HTTP error occurred: {http_err}"}
    except Timeout:
        # Handle timeout error
        return {"error": "Request timed out."}
    except RequestException as req_err:
        # Handle other request exceptions
        return {"error": f"Request exception occurred: {req_err}"}
    except Exception as err:
        # Handle any other unexpected errors
        return {"error": f"An unexpected error occurred: {err}"}
        
def make_request_for_content(url):
    """
    Make a request to an API if the API call returns content other than in JSON format.
    
    Args:
    - url (str): The url of the API.
    
    Returns:
    - string: Text in any format containing either the response data or an error message.
    """
    try:
        response = requests.get(url)
        return response.content
    except HTTPError as http_err:
        # Handle HTTP error
        return {"error": f"HTTP error occurred: {http_err}"}
    except Timeout:
        # Handle timeout error
        return {"error": "Request timed out."}
    except RequestException as req_err:
        # Handle other request exceptions
        return {"error": f"Request exception occurred: {req_err}"}
    except Exception as err:
        # Handle any other unexpected errors
        return {"error": f"An unexpected error occurred: {err}"}
      
        


from .. helperFuncs import make_request as mr

class SpaceflightNews:
    
    """
    A class to interact with the Spaceflight News API.
    
    Attributes:
    - base_url: The base URL of the Spaceflight News API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://api.spaceflightnewsapi.net/v4/"
        self.about = "The Spaceflight News API (SNAPI) is a product by The Space Devs (TSD). It's the most complete and up-to-date spaceflight news API currently available."
        
    def get_docs_url(self):
        """
        Returns the url for the Spaceflight News API documentation.
        
        Args:
        - None
        
        Returns:
        - string: URL for API documentation.
        """
        return "https://api.spaceflightnewsapi.net/v4/docs"
        
    def get_articles(self, limit=10, offset=10, search=None, summary_contains=None, title_contains=None):
        """
        Returns a list of spaceflight articles fitting the given parameters.
        
        Args:
        - limit (int): Number of results to return per page. Defaults to 10.
        - offset (int): The initial index from which to return the results. Defaults to 10.
        - search (str): Search for documents with a specific phrase in the title or summary. Defaults to None.
        - summary_contains (str): Search for all documents with a specific phrase in the summary. Defaults to None.
        - title_contains (str): Search for all documents with a specific phrase in the title. Defaults to None.
        
        Returns:
        - dict: List of spaceflight articles matching the given parameters.
        """
        if search is not None and summary_contains is not None and title_contains is not None:
            endpoint = f"articles?limit={limit}&offset={offset}&search={search}&summary_contains={summary_contains}&title_contains={title_contains}"
        elif search is not None and summary_contains is not None:
            endpoint = f"articles?limit={limit}&offset={offset}&search={search}&summary_contains={summary_contains}"
        elif search is not None and title_contains is not None:
            endpoint = f"articles?limit={limit}&offset={offset}&search={search}&title_contains={title_contains}"
        elif summary_contains is not None and title_contains is not None:
            endpoint = f"articles?limit={limit}&offset={offset}&summary_contains={summary_contains}&title_contains={title_contains}"
        elif search is not None:
            endpoint = f"articles?limit={limit}&offset={offset}&search={search}"
        elif summary_contains is not None:
            endpoint = f"articles?limit={limit}&offset={offset}&summary_contains={summary_contains}"
        elif title_contains is not None:
            endpoint = f"articles?limit={limit}&offset={offset}&title_contains={title_contains}"
        else:
            endpoint = "articles"
        
        return mr.make_request(self.base_url + endpoint)
        
    def get_article_by_id(self, ID):
        """
        Returns a specific article.
        
        Args:
        - ID (int): A unique integer value identifying this article.
        
        Returns:
        - dict: A dictionary containing the article.
        """
        endpoint = f"articles/{ID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_blogs(self, limit=10, offset=10, search=None, summary_contains=None, title_contains=None):
        """
        Returns a list of spaceflight blogs fitting the given parameters.
        
        Args:
        - limit (int): Number of results to return per page. Defaults to 10.
        - offset (int): The initial index from which to return the results. Defaults to 10.
        - search (str): Search for documents with a specific phrase in the title or summary. Defaults to None.
        - summary_contains (str): Search for all documents with a specific phrase in the summary. Defaults to None.
        - title_contains (str): Search for all documents with a specific phrase in the title. Defaults to None.
        
        Returns:
        - dict: List of spaceflight blogs matching the given parameters.
        """
        if search is not None and summary_contains is not None and title_contains is not None:
            endpoint = f"blogs?limit={limit}&offset={offset}&search={search}&summary_contains={summary_contains}&title_contains={title_contains}"
        elif search is not None and summary_contains is not None:
            endpoint = f"blogs?limit={limit}&offset={offset}&search={search}&summary_contains={summary_contains}"
        elif search is not None and title_contains is not None:
            endpoint = f"blogs?limit={limit}&offset={offset}&search={search}&title_contains={title_contains}"
        elif summary_contains is not None and title_contains is not None:
            endpoint = f"blogs?limit={limit}&offset={offset}&summary_contains={summary_contains}&title_contains={title_contains}"
        elif search is not None:
            endpoint = f"blogs?limit={limit}&offset={offset}&search={search}"
        elif summary_contains is not None:
            endpoint = f"blogs?limit={limit}&offset={offset}&summary_contains={summary_contains}"
        elif title_contains is not None:
            endpoint = f"blogs?limit={limit}&offset={offset}&title_contains={title_contains}"
        else:
            endpoint = "blogs"
        
        return mr.make_request(self.base_url + endpoint)
        
    def get_blog_by_id(self, ID):
        """
        Returns a specific blog.
        
        Args:
        - ID (int): A unique integer value identifying this blog.
        
        Returns:
        - dict: A dictionary containing the blog.
        """
        endpoint = f"blogs/{ID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_info(self):
        """
        Returns information containing API version and news sites featured.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing the version number and a list of features news sites.
        """
        endpoint = "info"
        return mr.make_request(self.base_url+endpoint)
        
    def get_reports(self, limit=10, offset=10, search=None, summary_contains=None, title_contains=None):
        """
        Returns a list of spaceflight reports fitting the given parameters.
        
        Args:
        - limit (int): Number of results to return per page. Defaults to 10.
        - offset (int): The initial index from which to return the results. Defaults to 10.
        - search (str): Search for documents with a specific phrase in the title or summary. Defaults to None.
        - summary_contains (str): Search for all documents with a specific phrase in the summary. Defaults to None.
        - title_contains (str): Search for all documents with a specific phrase in the title. Defaults to None.
        
        Returns:
        - dict: List of spaceflight reports matching the given parameters.
        """
        if search is not None and summary_contains is not None and title_contains is not None:
            endpoint = f"reports?limit={limit}&offset={offset}&search={search}&summary_contains={summary_contains}&title_contains={title_contains}"
        elif search is not None and summary_contains is not None:
            endpoint = f"reports?limit={limit}&offset={offset}&search={search}&summary_contains={summary_contains}"
        elif search is not None and title_contains is not None:
            endpoint = f"reports?limit={limit}&offset={offset}&search={search}&title_contains={title_contains}"
        elif summary_contains is not None and title_contains is not None:
            endpoint = f"reports?limit={limit}&offset={offset}&summary_contains={summary_contains}&title_contains={title_contains}"
        elif search is not None:
            endpoint = f"reports?limit={limit}&offset={offset}&search={search}"
        elif summary_contains is not None:
            endpoint = f"reports?limit={limit}&offset={offset}&summary_contains={summary_contains}"
        elif title_contains is not None:
            endpoint = f"reports?limit={limit}&offset={offset}&title_contains={title_contains}"
        else:
            endpoint = "reports"
        
        return mr.make_request(self.base_url + endpoint)
        
    def get_report_by_id(self, ID):
        """
        Returns a specific report.
        
        Args:
        - ID (int): A unique integer value identifying this report.
        
        Returns:
        - dict: A dictionary containing the report.
        """
        endpoint = f"reports/{ID}"
        return mr.make_request(self.base_url+endpoint)
     
    
   
      
        



from .. helperFuncs import make_request as mr

class JokeAPI:
    """
    A class to interact with the JokeAPI API.
    
    Attributes:
    - base_url: Base URL for interacting with the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://v2.jokeapi.dev/joke/"
        self.about = "JokeAPI is a REST API that serves uniformly and well formatted jokes. It can be used without any API token, membership, registration or payment. It supports a variety of filters that can be applied to get just the right jokes you need."
        
    def get_docs_url(self):
        """
        Returns the URL for the JokeAPI documentation.
        
        Args:
        - None
        
        Returns:
        - string: The url for the API documentation.
        """
        return "https://sv443.net/jokeapi/v2/"
        
    def get_joke(self, category=None, lang=None, flags=None, joke_type=None, search_string=None, amount=None):
        """
        Returns joke(s) matching the given parameters.
    
        Args:
        - category (str): Joke category. Defaults to Any. Possible non-default values are Programming, Misc, Dark, Pin, Spooky, and Christmas.
        - lang (str): Language of the joke. Defaults to en (English). Possible other values are cs (Czech), de (German), es (Spanish), fr (French), and pt (Portuguese).
        - flags (str): Optional flags to blacklist. Possible values are nsfw, religious, political, racist, sexist, and explicit.
        - joke_type (str): Optional parameter for either single or twopart joke(s). Defaults to None.
        - search_string (str): Optional search string contained in desired joke(s). Defaults to None.
        - amount (int): Number of jokes returned. Defaults to 1.
    
        Returns:
        - dict: A dictionary containing joke(s) matching given parameters.
        """
        endpoint = ""
    
        if category is not None:
            endpoint += f"{category}"
        else:
            endpoint += "Any"
        
        if lang is not None:
            endpoint += f"?lang={lang}"
        
        if flags is not None:
            if "?" not in endpoint:
                endpoint += f"?blacklistFlags={flags}"
            else:
                endpoint += f"&blacklistFlags={flags}"
            
        if joke_type is not None:
            if "?" not in endpoint:
                endpoint += f"?type={joke_type}"
            else:
                endpoint += f"&type={joke_type}"
            
        if search_string is not None:
            if "?" not in endpoint:
                endpoint += f"?contains={search_string}"
            else:
                endpoint += f"&contains={search_string}"
            
        if amount is not None:
            if "?" not in endpoint:
                endpoint += f"?amount={amount}"
            else:
                endpoint += f"&amount={amount}"
        
        return mr.make_request(self.base_url+endpoint)

        
      